@@ -245,6 +245,9 @@ window.CropDuster = {};
                 return;
             }
             CropDuster.setThumbnails(prefix, data.crop.thumbs);
+
+            // add preview_url from context data onto input field for preview rendering
+            $("#id_" + prefix).data('previewUrl', data.preview_url);
             CropDuster.createThumbnails(prefix);
             $(document).trigger('cropduster:update', [prefix, data]);
         },
@@ -352,10 +355,6 @@ window.CropDuster = {};
                 if (data.tmpFile) {
                     name += "_tmp";
                 }
-                var url = [CropDuster.mediaUrl, path, name + ext].join('/');
-                // This is in place of a negative lookbehind. It replaces all
-                // double slashes that don't follow a colon.
-                url = url.replace(/(:)?\/+/g, function($0, $1) { return $1 ? $0 : '/'; });
                 thumbData[slug] = {
                     'image_url': data.url,
                     'size_slug': slug,
@@ -372,6 +371,14 @@ window.CropDuster = {};
                     $thumb.html($thumb.html() + $.render.cropdusterImage(thumbData[size.name]));
                 }
             });
+            preview_img = {
+                'image_url': data.previewUrl,
+                'size_slug': '',
+                // use width and height of first thumb for admin preview
+                'width': data.sizes[0].w,
+                'height': data.sizes[0].h,
+            }
+            $thumb.html($.render.cropdusterImage(preview_img) + $thumb.html());
         },
 
         removeSize: function(prefix, sizeName) {
