@@ -62,6 +62,13 @@
     function setOptions(opt) //{{{
     {
       if (typeof(opt) !== 'object') opt = {};
+
+      if (opt.aspectRatio) {
+        opt.minAspectRatio = opt.aspectRatio;
+        opt.maxAspectRatio = opt.aspectRatio;
+        delete opt.aspectRatio;
+      }
+
       options = $.extend(options, opt);
 
       $.each(['onChange','onSelect','onRelease','onDblClick'],function(i,e) {
@@ -84,6 +91,7 @@
 
       Coords.setPressed(Coords.getCorner(opp));
       Coords.setCurrent(opc);
+      Coords.setWidthHeight(fc.w, fc.h);
 
       Tracker.activateHandlers(dragmodeHandler(mode, fc), doneSelect, touch);
     }
@@ -91,7 +99,7 @@
     function dragmodeHandler(mode, f) //{{{
     {
       return function (pos) {
-        if (!options.aspectRatio) {
+        if (!options.minAspectRatio || !options.maxAspectRatio) {
           switch (mode) {
           case 'e':
             pos[1] = f.y2;
@@ -122,7 +130,7 @@
             break;
           }
         }
-        Coords.setCurrent(pos);
+        Coords.setCurrent(pos, mode);
         Selection.update();
       };
     }
@@ -227,15 +235,11 @@
     //}}}
     function newSelection(e) //{{{
     {
-      var c = Coords.getFixed();
-      if ((c.w > options.minSelect[0]) && c.h > options.minSelect[1]) {
-        return false;
-      }
       if (options.disabled) {
-        return false;
+        return;
       }
       if (!options.allowSelect) {
-        return false;
+        return;
       }
       btndown = true;
       docOffset = getPos($img);
@@ -463,7 +467,7 @@
           y1 = 0,
           x2 = 0,
           y2 = 0,
-          ox, oy;
+          ox, oy, iw, ih;
 
       function setPressed(pos) //{{{
       {
@@ -472,13 +476,54 @@
         y2 = y1 = pos[1];
       }
       //}}}
-      function setCurrent(pos) //{{{
+      function setCurrent(pos, mode) //{{{
       {
         pos = rebound(pos);
+        var c1, c2, old_x1 = x1, old_y1 = y1;
+        if (typeof mode != 'undefined') {
+          c1 = Coords.getFixed();
+          x1 = old_x1;
+          y1 = old_y1;
+        }
         ox = pos[0] - x2;
         oy = pos[1] - y2;
         x2 = pos[0];
         y2 = pos[1];
+        if (typeof mode == 'undefined') {
+          return;
+        }
+        c2 = Coords.getFixed();
+        x1 = old_x1;
+        y1 = old_y1;
+        if (c1.w == c2.w && c1.h == c2.h) {
+          return;
+        }
+        if (ox !== 0 && oy !== 0) {
+          return;
+        }
+        var aspect = c2.w / c2.h,
+            minAspect = options.minAspectRatio,
+            maxAspect = options.maxAspectRatio;
+        if (minAspect && aspect <= minAspect || maxAspect && aspect >= maxAspect) {
+          switch (mode) {
+            case 'n':
+            case 's':
+              var dx = c1.x2 - c2.x2;
+              x1 += dx / 2;
+              break;
+            case 'e':
+            case 'w':
+              var dy = c1.y2 - c2.y2;
+              y1 += dy / 2;
+              break;
+          }
+        }
+      }
+      //}}}
+      function setWidthHeight(w, h) //{{{
+      {
+        iw = w;
+        ih = h;
       }
       //}}}
       function getOffset() //{{{
@@ -528,11 +573,12 @@
       //}}}
       function getFixed() //{{{
       {
-        if (!options.aspectRatio) {
+        if (!options.minAspectRatio && !options.maxAspectRatio) {
           return getRect();
         }
         // This function could use some optimization I think...
-        var aspect = options.aspectRatio,
+        var minAspect = options.minAspectRatio,
+            maxAspect = options.maxAspectRatio,
             min_x = options.minSize[0] / xscale,
             
             
@@ -544,7 +590,32 @@
             rwa = Math.abs(rw),
             rha = Math.abs(rh),
             real_ratio = rwa / rha,
-            xx, yy, w, h;
+            xx, yy, w, h, aspect,
+            vertical = function() {
+                yy = y2;
+                w = rha * aspect;
+                xx = rw < 0 ? x1 - w : w + x1;
+
+                if (xx < 0) {
+                  xx = 0;
+                } else if (xx > boundx) {
+                  xx = boundx;
+                }
+                h = Math.abs((xx - x1) / aspect);
+                yy = rh < 0 ? y1 - h : h + y1;
+            },
+            horizontal = function() {
+                xx = x2;
+                h = rwa / aspect;
+                yy = rh < 0 ? y1 - h : y1 + h;
+                if (yy < 0) {
+                  yy = 0;
+                } else if (yy > boundy) {
+                  yy = boundy;
+                }
+                w = Math.abs((yy - y1) * aspect);
+                xx = rw < 0 ? x1 - w : w + x1;
+            };
 
         if (max_x === 0) {
           max_x = boundx * 10;
@@ -552,35 +623,16 @@
         if (max_y === 0) {
           max_y = boundy * 10;
         }
-        if (real_ratio < aspect) {
-          yy = y2;
-          w = rha * aspect;
-          xx = rw < 0 ? x1 - w : w + x1;
-
-          if (xx < 0) {
-            xx = 0;
-            h = Math.abs((xx - x1) / aspect);
-            yy = rh < 0 ? y1 - h : h + y1;
-          } else if (xx > boundx) {
-            xx = boundx;
-            h = Math.abs((xx - x1) / aspect);
-            yy = rh < 0 ? y1 - h : h + y1;
-          }
+        if (minAspect && real_ratio < minAspect) {
+          aspect = minAspect;
+        } else if (maxAspect && real_ratio > maxAspect) {
+          aspect = maxAspect;
         } else {
-          xx = x2;
-          h = rwa / aspect;
-          yy = rh < 0 ? y1 - h : y1 + h;
-          if (yy < 0) {
-            yy = 0;
-            w = Math.abs((yy - y1) * aspect);
-            xx = rw < 0 ? x1 - w : w + x1;
-          } else if (yy > boundy) {
-            yy = boundy;
-            w = Math.abs(yy - y1) * aspect;
-            xx = rw < 0 ? x1 - w : w + x1;
-          }
+          return getRect();
         }
 
+        Math.abs(rwa - iw) > Math.abs(rha - ih) ? horizontal() : vertical();
+
         // Magic %-)
         if (xx > x1) { // right side
           if (xx - x1 < min_x) {
@@ -633,7 +685,7 @@
         if (p[0] > boundx) p[0] = boundx;
         if (p[1] > boundy) p[1] = boundy;
 
-        return [p[0], p[1]];
+        return [Math.round(p[0]), Math.round(p[1])];
       }
       //}}}
       function flipCoords(x1, y1, x2, y2) //{{{
@@ -659,11 +711,11 @@
             ysize = y2 - y1,
             delta;
 
-        if (xlimit && (Math.abs(xsize) > xlimit)) {
-          x2 = (xsize > 0) ? (x1 + xlimit) : (x1 - xlimit);
+        if (xlimit && (Math.abs(xsize) > xlimit / xscale)) {
+          x2 = (xsize > 0) ? (x1 + xlimit / xscale) : (x1 - xlimit / xscale);
         }
-        if (ylimit && (Math.abs(ysize) > ylimit)) {
-          y2 = (ysize > 0) ? (y1 + ylimit) : (y1 - ylimit);
+        if (ylimit && (Math.abs(ysize) > ylimit / yscale)) {
+          y2 = (ysize > 0) ? (y1 + ylimit / yscale) : (y1 - ylimit / yscale);
         }
 
         if (ymin / yscale && (Math.abs(ysize) < ymin / yscale)) {
@@ -730,6 +782,7 @@
         flipCoords: flipCoords,
         setPressed: setPressed,
         setCurrent: setCurrent,
+        setWidthHeight: setWidthHeight,
         getOffset: getOffset,
         moveOffset: moveOffset,
         getCorner: getCorner,
@@ -1664,7 +1717,8 @@
     handleOpacity: 0.5,
     handleSize: null,
 
-    aspectRatio: 0,
+    minAspectRatio: 0,
+    maxAspectRatio: 0,
     keySupport: true,
     createHandles: ['n','s','e','w','nw','ne','se','sw'],
     createDragbars: ['n','s','e','w'],
