@@ -1,29 +1,22 @@
 from __future__ import division
 
-import six
-
-from six.moves import xrange
-
+from io import BytesIO
 import os
-import tempfile
 import warnings
 import math
+from distutils.version import LooseVersion
 
 import PIL.Image
+from PIL import ImageFile, JpegImagePlugin
 
-try:
-    import numpy
-except ImportError:
-    numpy = None
+from django.core.files.storage import default_storage
+from django.utils import six
+from django.utils.six.moves import xrange
 
-try:
-    import scipy
-except ImportError:
-    scipy = None
+from cropduster.settings import (
+    get_jpeg_quality, JPEG_SAVE_ICC_SUPPORTED, CROPDUSTER_GIFSICLE_PATH)
 
-from cropduster.settings import get_jpeg_quality
-
-from .images2gif import read_gif, write_gif
+from .gifsicle import GifsicleImage
 
 
 __all__ = (
@@ -32,6 +25,13 @@ __all__ = (
     'process_image', 'smart_resize')
 
 
+# workaround for https://github.com/python-pillow/Pillow/issues/1138
+# without this hack, pillow misidenfifies some jpeg files as "mpo" files
+JpegImagePlugin._getmp = lambda x: None  # noqa
+
+ImageFile.MAXBLOCK = 4096 * 4096 * 8
+
+
 IMAGE_EXTENSIONS = {
     "ARG":  ".arg",   "BMP":  ".bmp",   "BUFR": ".bufr",  "CUR":  ".cur",   "DCX":  ".dcx",
     "EPS":  ".ps",    "FITS": ".fit",   "FLI":  ".fli",   "FPX":  ".fpx",   "GBR":  ".gbr",
@@ -40,6 +40,7 @@ IMAGE_EXTENSIONS = {
     "MSP":  ".msp",   "Palm": ".palm",  "PCD":  ".pcd",   "PCX":  ".pcx",   "PDF":  ".pdf",
     "PNG":  ".png",   "PPM":  ".ppm",   "PSD":  ".psd",   "SGI":  ".rgb",   "SUN":  ".ras",
     "TGA":  ".tga",   "TIFF": ".tiff",  "WMF":  ".wmf",   "XBM":  ".xbm",   "XPM":  ".xpm",
+    "MPO":  ".jpg",  # Pillow mislabels some jpeg images as MPO files
 }
 
 
@@ -121,66 +122,42 @@ def is_animated_gif(im):
 
 
 def has_animated_gif_support():
-    return bool(numpy and scipy)
+    return bool(CROPDUSTER_GIFSICLE_PATH)
 
 
 def process_image(im, save_filename=None, callback=lambda i: i, nq=0, save_params=None):
     is_animated = is_animated_gif(im)
     images = [im]
 
-    dispose = None
-
     if is_animated:
-        if not has_animated_gif_support():
+        if not CROPDUSTER_GIFSICLE_PATH:
             warnings.warn(
                 u"This server does not have animated gif support; your uploaded image "
                 u"has been made static.")
         else:
-            filename = getattr(im, 'filename', None)
-            if not filename or not os.path.exists(filename):
-                temp_file = tempfile.NamedTemporaryFile(suffix='.gif')
-                filename = temp_file.name
-                im.save(filename)
-
-            contents = b''
-            with open(filename, mode='rb') as f:
-                contents += f.read()
-
-            try:
-                graphics_control_ext_offset = contents.index('\x21\xF9\x04')
-            except ValueError:
-                pass
-            else:
-                try:
-                    dispose_byte = contents[graphics_control_ext_offset + 3]
-                except IndexError:
-                    pass
-                else:
-                    dispose = six.byte2int(dispose_byte) >> 2
-
-            images = read_gif(filename, as_numpy=False)
+            images = [GifsicleImage(im)]
 
     new_images = [callback(i) for i in images]
 
-    if len(images) > 1 and not save_filename:
+    if is_animated and not save_filename:
         raise Exception("Animated gifs must be saved on each processing.")
 
     if save_filename:
-        # Only true if animated gif supported and multiple frames in image
-        if is_animated and len(images) > 1:
-            duration_ms = im.info.get('duration') or 100
-            duration = float(duration_ms) / 1000.0
-            repeat = True
-            if im.info.get('loop', 0) != 0:
-                repeat = im.info['loop']
-            write_gif(save_filename, new_images, duration=duration, repeat=repeat, nq=nq, dispose=dispose)
-        else:
-            save_params = save_params or {}
-            if im.format == 'JPEG':
-                save_params.setdefault('quality', get_jpeg_quality(im.size[0], im.size[1]))
-            new_images[0].save(save_filename, **save_params)
-
-        return PIL.Image.open(save_filename)
+        save_params = save_params or {}
+        if im.format == 'JPEG':
+            save_params.setdefault('quality', get_jpeg_quality(new_images[0].size[0], new_images[0].size[1]))
+        if im.format in ('JPEG', 'PNG') and JPEG_SAVE_ICC_SUPPORTED:
+            save_params.setdefault('icc_profile', im.info.get('icc_profile'))
+        img = new_images[0]
+        buf = BytesIO()
+        img.save(buf, format=im.format, **save_params)
+        with default_storage.open(save_filename, 'wb') as f:
+            f.write(buf.getvalue())
+        with default_storage.open(save_filename, mode='rb') as f:
+            content = f.read()
+            pil_image = PIL.Image.open(BytesIO(content))
+        pil_image.filename = save_filename
+        return pil_image
 
     return new_images[0]
 
@@ -199,6 +176,12 @@ def smart_resize(im, final_w, final_h):
         # If the image is already the right size, don't change it
         return im
 
+    # Pillow 2.7.0 greatly improved the bicubic resize algorithm, which makes
+    # our multiple-step resizing unnecessary
+    pillow_version = getattr(PIL, 'PILLOW_VERSION', None)
+    if pillow_version and LooseVersion(pillow_version) >= LooseVersion('2.7.0'):
+        return im.resize((final_w, final_h), PIL.Image.BICUBIC)
+
     # Attempt to resize the image 1/8, 2/8, such that it is at least 1.5x bigger
     # than the final size
     # (Libjpg-Turbo has optimizations for resizing images by a ratio of eights)
